/******************************************************************************
*                                                                             *
* Prompt Helper - A script that helps to customise the shell prompt           *
* Copyright (C) 2011  Fabien LOISON <http://misc.flogisoft.com/>              *
*                                                                             *
* This program is free software: you can redistribute it and/or modify        *
* it under the terms of the GNU Affero General Public License as published by *
* the Free Software Foundation, either version 3 of the License, or           *
* (at your option) any later version.                                         *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU Affero General Public License for more details.                         *
*                                                                             *
* You should have received a copy of the GNU Affero General Public License    *
* along with this program.  If not, see <http://www.gnu.org/licenses/>.       *
*                                                                             *
******************************************************************************/



var ph_div = document.getElementById("ph_prompt_helper");
var ph_titlebar = 1;
var ph_prev = null;
var ph_input_div = null;
var ph_input = null;



var BSCHARS = {
	'd': "Mon Jun 13",
	't': "22:42:01",
	'T': "10:42:01",
	'@': "10:42 PM",
	'h': "hostname",
	'H': "hostname.domain",
	'j': "0",
	'l': "tty1",
	's': "bash",
	'u': "username",
	'v': "4.1",
	'V': "4.1.5",
	'w': "~/Documents",
	'W': "Documents",
	'!': "538",
	'$': "$",
	'[': "",
	']': "",
	'\\': "\\",
	};

var ENV = {
	'$HOME': "/home/username",
	'$USER': "username",
	'$PWD': "/home/username/Documents",
	'$PS1': "-- It's an inception? ;p --",
	'$DISPLAY': "-- You really want to display that? oO --",
	'$0': "bash",
	'$_': "TERM",
	'$#': "TERM",
	'$?': "0",
	};

var FGCOLORS = {
	'30': "#000000",
	'31': "#cd0000",
	'32': "#00cd00",
	'33': "#cdcd00",
	'34': "#1e90ff",
	'35': "#cd00cd",
	'36': "#00cdcd",
	'37': "#e5e5e5",
	'39': "#ffffff", //default
	'90': "#4c4c4c",
	'91': "#ff0000",
	'92': "#00ff00",
	'93': "#ffff00",
	'94': "#4682b4",
	'95': "#ff00ff",
	'96': "#00ffff",
	'97': "#ffffff",
	};

var BGCOLORS = {
	'40': "#000000",
	'41': "#cd0000",
	'42': "#00cd00",
	'43': "#cdcd00",
	'44': "#1e90ff",
	'45': "#cd00cd",
	'46': "#00cdcd",
	'47': "#e5e5e5",
	'49': "#000000", //default
	'100': "#4c4c4c",
	'101': "#ff0000",
	'102': "#00ff00",
	'103': "#ffff00",
	'104': "#4682b4",
	'105': "#ff00ff",
	'106': "#00ffff",
	'107': "#ffffff",
	};



function _clear_inner(e) {
	while(e.firstChild) {
		e.removeChild(e.firstChild);
	}
}



function _mk_ph_node(str, fg, bg, bold, underline) {
	node = document.createElement("span");
	node.style.color = fg;
	node.style.background = bg;
	if (bold) {
		node.style.fontWeight = "bold";
	}
	if (underline) {
		node.style.textDecoration = "underline";
	}
	if (str == " ") {
		node.appendChild(document.createTextNode("\u00a0"));
	} else {
		node.appendChild(document.createTextNode(str));
	}
	return node;
}



function _ph_input_width() {
	ph_input.style.width = ph_div.offsetWidth - 70 + "px";	
}



function ph_init() {
	//#prompt_helper style
	ph_div.style.border = "#c0c0c0 solid 3px";
	ph_div.style.padding = "0px";
	ph_div.style.background = "#000000";
	ph_div.style.color = "#ffffff";
	ph_div.style.fontFamily = "monospace";
	ph_div.style.fontSize = "9pt";
	//Clear the content of #prompt_helper
	_clear_inner(ph_div);
	//Add #ph_titlebar, and set its style and content
	ph_titlebar = document.createElement("div");
	ph_titlebar.setAttribute("id", "ph_titlebar");
	ph_titlebar.style.padding = "0 0 3px 0";
	ph_titlebar.style.background = "#c0c0c0";
	ph_titlebar.style.color = "#000000";
	ph_titlebar.style.fontFamily = "sans-serif";
	ph_titlebar.style.fontWeight = "bold";
	ph_titlebar.style.minHeight = "16px";
	ph_titlebar.style.overflow = "hidden";
	ph_titlebar.style.whiteSpace = "nowrap";
	ph_titlebar.appendChild(document.createTextNode("Prompt Helper"));
	ph_div.appendChild(ph_titlebar);
	//Add #ph_prev, and set its style and content
	ph_prev = document.createElement("div");
	ph_prev.setAttribute("id", "ph_prev");
	ph_prev.style.padding = "3px 5px 5px 5px";
	ph_prev.style.minHeight = "16px";
	ph_prev.style.overflow = "hidden";
	ph_prev.appendChild(document.createTextNode("Prompt Preview"));
	ph_div.appendChild(ph_prev);
	//Add #ph_input_div, and set its style and content
	ph_input_div = document.createElement("div");
	ph_input_div.setAttribute("id", "ph_input_div");
	ph_input_div.style.padding = "3px 5px";
	ph_input_div.style.fontWeight = "bold";
	ph_input_div.appendChild(document.createTextNode("PS1=\""));
	ph_div.appendChild(ph_input_div);
	//Add #ph_input
	ph_input = document.createElement("input");
	ph_input.setAttribute("type", "text");
	ph_input.setAttribute("name", "ph_input");
	ph_input.setAttribute("id", "ph_input");
	//ph_input.setAttribute("value", "\\t - \\T - \\@ - \\d $ ");
	ph_input.setAttribute("value", "\\e[1;32m\\u@\\h\\e[0;1m:\\e[0;36m\\w\\e[0m\\$ ");
	ph_input.style.border = "#444 dashed 1px";
	ph_input.style.background = "#222";
	ph_input.style.color = "#aaa";
	ph_input.style.fontFamily = "monospace";
	ph_input.style.fontSize = "9pt";
	ph_input.addEventListener("keyup",
		function() {
			ph_parse_prompt(ph_input.value);
		},
		true
		);
	ph_input.addEventListener("change",
		function() {
			ph_parse_prompt(ph_input.value);
		},
		true
		);
	ph_input.addEventListener("mouseup",
		function() {
			ph_parse_prompt(ph_input.value);
		},
		true
		);
	ph_input_div.appendChild(ph_input);
	ph_input_div.appendChild(document.createTextNode("\""));
}



function ph_parse_title(title_str) {
	var mode = "char"; //char, bs, var
	var buff = "";
	var result = "";
	//Make the prompt more easy to parse
	title_str = title_str.replace(/([^\\]|^)\$\{(.*)\$(.*)\}/gi, "$1\${$2$3}");
	title_str = title_str.replace(/([^\\]|^)\$\$/gi, "$1"+"1337");
	title_str = title_str.replace(/([^\\]|^)\$([a-z_][a-z0-9_]*|[0-9]|\?|#|@|\*)/gi, "$1\${$2}");
	//
	for (i in title_str) {
		switch (mode) {

			case "char":
				if (title_str[i] == "\\") {
					mode = "bs";
				} else if (title_str[i] == "$") {
					mode = "var";
					buff = "";
				} else {
					result += title_str[i];
				}
				break;

			case "var":
				if (title_str[i] == "{" && buff == "") {
					buff = "$";
				} else if (title_str[i] != "{" && buff == "") {
					result += "$" + title_str[i];
					mode = "char";
				} else if (title_str[i] == "}") {
					if (buff in ENV) {
						result += ENV[buff];
					}
					mode = "char";
				} else {
					buff += title_str[i];
				}
				break;

			case "bs":
				if (title_str[i] in BSCHARS) {
					result += BSCHARS[title_str[i]];
					mode = "char";
				} else {
					result += "\\" + title_str[i];
					mode = "char";
				}
				break;
		}
	}
	return result;
}



function ph_parse_prompt(prompt_str) {
	var mode = "char"; //char, bs, var, ESC, CSI, OSC, OSC-T
	var fg_color = "#fff";
	var bg_color = "#000";
	var attr_bold = false;
	var attr_underline = false;
	//Clear the preview div and set the default title
	_clear_inner(ph_prev);
	_clear_inner(ph_titlebar);
	ph_titlebar.appendChild(document.createTextNode("Prompt Helper"))
	//Make the prompt more easy to parse
	prompt_str = prompt_str.replace(/\\E/g, "\\e");
	prompt_str = prompt_str.replace(/\\033/g, "\\e");
	prompt_str = prompt_str.replace(/\\007/g, "\u0007");
	prompt_str = prompt_str.replace(/\\a/g, "\u0007");
	prompt_str = prompt_str.replace(/([^\\]|^)\$\{(.*)\$(.*)\}/gi, "$1\${$2$3}");
	prompt_str = prompt_str.replace(/([^\\]|^)\$\$/gi, "$1"+"1337");
	prompt_str = prompt_str.replace(/([^\\]|^)\$([a-z_][a-z0-9_]*|[0-9]|\?|#|@|\*)/gi, "$1\${$2}");
	//
	for (i in prompt_str) {
		switch (mode) {

			case "char":
				if (prompt_str[i] == "\\") {
					mode = "bs";
				} else if (prompt_str[i] == "$") {
					mode = "var";
					buff = "";
				} else {
					ph_prev.appendChild(_mk_ph_node(
								prompt_str[i],
								fg_color,
								bg_color,
								attr_bold,
								attr_underline
								));
				}
				break;

			case "var":
				if (prompt_str[i] == "{" && buff == "") {
					buff = "$";
				} else if (prompt_str[i] != "{" && buff == "") {
					ph_prev.appendChild(_mk_ph_node(
								"$"+prompt_str[i],
								fg_color,
								bg_color,
								attr_bold,
								attr_underline
								));
					mode = "char";
				} else if (prompt_str[i] == "}") {
					if (buff in ENV) {
						ph_prev.appendChild(_mk_ph_node(
									ENV[buff],
									fg_color,
									bg_color,
									attr_bold,
									attr_underline
									));
					}
					mode = "char";
				} else {
					buff += prompt_str[i];
				}
				break;

			case "bs":
				if (prompt_str[i] in BSCHARS) {
					ph_prev.appendChild(_mk_ph_node(
								BSCHARS[prompt_str[i]],
								fg_color,
								bg_color,
								attr_bold,
								attr_underline
								));
					mode = "char";
				} else if (prompt_str[i] == "e") {
					mode = "ESC";
				} else if (prompt_str[i] == "r") {
					_clear_inner(ph_prev);
					mode = "char";
				} else if (prompt_str[i] == "n") {
					ph_prev.appendChild(document.createElement("br"));
					mode = "char";
				} else {
					ph_prev.appendChild(_mk_ph_node(
								"\\" + prompt_str[i],
								fg_color,
								bg_color,
								attr_bold,
								attr_underline
								));
					mode = "char";
				}
				break;

			case "ESC":
				if (prompt_str[i] == "[") { //CSI
					mode = "CSI";
					buff = "";
				} else if (prompt_str[i] == "]") { //OSC
					mode = "OSC";
					buff = "";
				} else {
					ph_prev.appendChild(_mk_ph_node(
								"\u001b" + prompt_str[i],
								fg_color,
								bg_color,
								attr_bold,
								attr_underline
								));
					mode = "char";
				}
				break;

			case "CSI":
				validattr = {"1":"", "2":"", "3":"", "4":"", "5":"",
				             "6":"", "7":"", "8":"", "9":"", "0":"",
							 ";":""}
				if (prompt_str[i] in validattr) {
					buff += prompt_str[i];
				} else if (prompt_str[i] == "m") {
					buff = buff.replace(/(38|48);5;([0-9]{1,3})/, "$1:5:$2"); //256 colors
					attr = buff.split(";")
					for (i in attr) {
						attr[i] = attr[i].replace(/^0+(.+)/, "$1");
						switch (attr[i]) {
							case "0": //Clear all attr
								fg_color = "#fff";
								bg_color = "#000";
								attr_bold = false;
								attr_underline = false;
								break;
							case "1": //Bold
								attr_bold = true;
								break;
							case "4": //underline
								attr_underline = true;
								break;
							case "7": //invert
								invtmp = fg_color;
								fg_color = bg_color;
								bg_color = invtmp;
								break;
							default:
								if (attr[i] in FGCOLORS) {
									fg_color = FGCOLORS[attr[i]];
								} else if (attr[i] in BGCOLORS) {
									bg_color = BGCOLORS[attr[i]];
								}
						}
					}
					mode = "char";
				} else {
					ph_prev.appendChild(_mk_ph_node(
								"\u001b[" + buff + prompt_str[i],
								fg_color,
								bg_color,
								attr_bold,
								attr_underline
								));
					mode = "char";
				}
				break;

			case "OSC":
				if (prompt_str[i] == "0" || prompt_str[i] == "2") {
					buff = prompt_str[i];
				} else if (prompt_str[i] == ";" && buff != "") {
					mode = "OSC-T";
					buff = "";
				} else {
					ph_prev.appendChild(_mk_ph_node(
								"\u001b]" + prompt_str[i],
								fg_color,
								bg_color,
								attr_bold,
								attr_underline
								));
					mode = "char";
				}
				break;

			case "OSC-T":
				if (prompt_str[i] == "\u0007") {
					_clear_inner(ph_titlebar);
					ph_titlebar.appendChild(document.createTextNode(
								ph_parse_title(buff) + " - Prompt Helper"))
						mode = "char";
				} else {
					buff += prompt_str[i];
				}
				break;
		}
	}
}



ph_init();
ph_parse_prompt(ph_input.getAttribute("value"));
setInterval("_ph_input_width()", 500);


